﻿
using DbContextLib;
using Microsoft.EntityFrameworkCore;
using StubbedContextLib;

class Program
{
    static void Main(string[] args)
    {

        try {
            using (LibraryContext db = new TrainingStubbedContext())
            {
                ActivityTests(db);

                DataSourceTests(db);

                AthleteTests(db);
            }
        }
        catch (Exception ex)
        {
            Console.WriteLine($"Une erreur s'est produite : {ex.Message}");
        }
    }
    static void ActivityTests(LibraryContext db)
    {
        Console.WriteLine("Accès à toutes les activités avec leurs fréquences cardiaques :");

        Console.WriteLine("Activités :");
        Console.WriteLine("---------------------------------");

        foreach (var activity in db.ActivitiesSet.Include(a => a.HeartRates))
        {
            Console.WriteLine($"\t{activity.IdActivity} - {activity.Type}, {activity.Date}, {activity.StartTime}, {activity.EndTime}, {activity.EffortFelt}, {activity.Variability}, {activity.Variance}, {activity.StandardDeviation}, {activity.Average}, {activity.Maximum}, {activity.Minimum}, {activity.AverageTemperature}, {activity.HasAutoPause}");

            Console.WriteLine("\tFréquences cardiaques :");
            Console.WriteLine("\t---------------------------------");

            foreach (var heartRate in activity.HeartRates)
            {
                Console.WriteLine($"\t\t{heartRate.IdHeartRate} - {heartRate.Altitude}, {heartRate.Time}, {heartRate.Temperature}, {heartRate.Bpm}, {heartRate.Longitude}, {heartRate.Latitude}");
            }
        }

        Console.WriteLine("---------------------------------\n");

        Console.WriteLine("Accès à l'activité d'id '2' :");
        Console.WriteLine("---------------------------------");

        foreach (var activity in db.ActivitiesSet.Where(a => a.IdActivity == 2).Include(a => a.HeartRates))
        {
            Console.WriteLine($"\t{activity.IdActivity} - {activity.Type}, {activity.Date}, {activity.StartTime}, {activity.EndTime}, {activity.EffortFelt}, {activity.Variability}, {activity.Variance}, {activity.StandardDeviation}, {activity.Average}, {activity.Maximum}, {activity.Minimum}, {activity.AverageTemperature}, {activity.HasAutoPause}");

            Console.WriteLine("\tFréquences cardiaques :");
            Console.WriteLine("\t---------------------------------");

            foreach (var heartRate in activity.HeartRates)
            {
                Console.WriteLine($"\t\t{heartRate.IdHeartRate} - {heartRate.Altitude}, {heartRate.Time}, {heartRate.Temperature}, {heartRate.Bpm}, {heartRate.Longitude}, {heartRate.Latitude}");
            }
        }
    }

    static void DataSourceTests(LibraryContext db)
    {
        Console.WriteLine("Accès à toutes les sources de données avec leurs activités :");

        Console.WriteLine("Sources de données :");
        Console.WriteLine("---------------------------------");

        foreach (var dataSource in db.DataSourcesSet.Include(ds => ds.Activities))
        {
            Console.WriteLine($"\t{dataSource.IdSource} - {dataSource.Model}");

            Console.WriteLine("\tActivités :");
            Console.WriteLine("\t---------------------------------");

            foreach (var activity in dataSource.Activities)
            {
                Console.WriteLine($"\t\t{activity.IdActivity} - {activity.Type}, {activity.Date}, {activity.StartTime}, {activity.EndTime}, {activity.EffortFelt}, {activity.Variability}, {activity.Variance}, {activity.StandardDeviation}, {activity.Average}, {activity.Maximum}, {activity.Minimum}, {activity.AverageTemperature}, {activity.HasAutoPause}");
            }

            Console.WriteLine("\tAthletes :");
            Console.WriteLine("\t---------------------------------");

            foreach (var athlete in dataSource.Athletes)
            {
                Console.WriteLine($"\t\t{athlete.IdAthlete} - {athlete.FirstName}, {athlete.LastName}, {athlete.DateOfBirth}, {athlete.Sexe}, {athlete.Weight}, {athlete.IsCoach}");
            }
        }

        Console.WriteLine("---------------------------------\n");

        Console.WriteLine("Accès à la source de données d'id '2' :");
        Console.WriteLine("---------------------------------");

        foreach (var dataSource in db.DataSourcesSet.Where(ds => ds.IdSource == 2).Include(ds => ds.Activities))
        {
            Console.WriteLine($"\t{dataSource.IdSource} - {dataSource.Model}");

            Console.WriteLine("\tActivités :");
            Console.WriteLine("\t---------------------------------");

            foreach (var activity in dataSource.Activities)
            {
                Console.WriteLine($"\t\t{activity.IdActivity} - {activity.Type}, {activity.Date}, {activity.StartTime}, {activity.EndTime}, {activity.EffortFelt}, {activity.Variability}, {activity.Variance}, {activity.StandardDeviation}, {activity.Average}, {activity.Maximum}, {activity.Minimum}, {activity.AverageTemperature}, {activity.HasAutoPause}");
            }

            Console.WriteLine("\tAthletes :");
            Console.WriteLine("\t---------------------------------");

            foreach (var athlete in dataSource.Athletes)
            {
                Console.WriteLine($"\t\t{athlete.IdAthlete} - {athlete.FirstName}, {athlete.LastName}, {athlete.DateOfBirth}, {athlete.Sexe}, {athlete.Weight}, {athlete.IsCoach}");
            }
        }

        Console.WriteLine("---------------------------------\n");
    }

    static void AthleteTests(LibraryContext db)
    {
        Console.WriteLine("Accès à tous les athlètes avec leurs statistiques :");

        Console.WriteLine("Athlètes :");
        Console.WriteLine("---------------------------------");

        // ! Pas oublier de faire tous les includes necessaire !
        // ? Mais comment ?
        foreach (var athlete in db.AthletesSet.Include(a => a.Statistics))
        {
            Console.WriteLine($"\t{athlete.IdAthlete} - {athlete.FirstName}, {athlete.LastName}, {athlete.DateOfBirth}, {athlete.Sexe}, {athlete.Weight}, {athlete.IsCoach}");

            Console.WriteLine("\tStatistiques :");
            Console.WriteLine("\t---------------------------------");

            foreach (var statistic in athlete.Statistics)
            {
                Console.WriteLine($"\t\t{statistic.IdStatistic} - {statistic.Date}, {statistic.AverageCaloriesBurned}, {statistic.AverageHeartRate}, {statistic.MaximumHeartRate}");
            }

            Console.WriteLine("\tActivités :");
            Console.WriteLine("\t---------------------------------");

            foreach (var activity in athlete.Activities)
            {
                Console.WriteLine($"\t\t{activity.IdActivity} - {activity.Type}, {activity.Date}, {activity.StartTime}, {activity.EndTime}, {activity.EffortFelt}, {activity.Variability}, {activity.Variance}, {activity.StandardDeviation}, {activity.Average}, {activity.Maximum}, {activity.Minimum}, {activity.AverageTemperature}, {activity.HasAutoPause}");
            }

            Console.WriteLine("\tEntraînements :");
            Console.WriteLine("\t---------------------------------");

            foreach (var training in athlete.Trainings)
            {
                Console.WriteLine($"\t\t{training.IdTraining} - {training.Date}, {training.Latitude}, {training.Longitude}, {training.Description}, {training.FeedBack}");
            }

            Console.WriteLine("\tNotifications :");
            Console.WriteLine("\t---------------------------------");

            foreach (var notification in athlete.Notifications)
            {
                Console.WriteLine($"\t\t{notification.IdNotif} - {notification.Date}, {notification.Statut}, {notification.Message}");
            }

            Console.WriteLine("\tSources de données :");
            Console.WriteLine("\t---------------------------------");

            Console.WriteLine("\t\t" + (athlete.DataSource?.Model ?? "Aucune source de données"));
        }

        Console.WriteLine("---------------------------------\n");

        Console.WriteLine("Accès à l'athlète d'id '2' :");
        Console.WriteLine("---------------------------------");

        foreach (var athlete in db.AthletesSet.Where(a => a.IdAthlete == 2).Include(a => a.Statistics))
        {
            Console.WriteLine($"\t{athlete.IdAthlete} - {athlete.FirstName}, {athlete.LastName}, {athlete.DateOfBirth}, {athlete.Sexe}, {athlete.Weight}, {athlete.IsCoach}");

            Console.WriteLine("\tStatistiques :");
            Console.WriteLine("\t---------------------------------");

            foreach (var statistic in athlete.Statistics)
            {
                Console.WriteLine($"\t\t{statistic.IdStatistic} - {statistic.Date}, {statistic.AverageCaloriesBurned}, {statistic.AverageHeartRate}, {statistic.MaximumHeartRate}");
            }

            Console.WriteLine("\tActivités :");
            Console.WriteLine("\t---------------------------------");

            foreach (var activity in athlete.Activities)
            {
                Console.WriteLine($"\t\t{activity.IdActivity} - {activity.Type}, {activity.Date}, {activity.StartTime}, {activity.EndTime}, {activity.EffortFelt}, {activity.Variability}, {activity.Variance}, {activity.StandardDeviation}, {activity.Average}, {activity.Maximum}, {activity.Minimum}, {activity.AverageTemperature}, {activity.HasAutoPause}");
            }

            Console.WriteLine("\tEntraînements :");
            Console.WriteLine("\t---------------------------------");

            foreach (var training in athlete.Trainings)
            {
                Console.WriteLine($"\t\t{training.IdTraining} - {training.Date}, {training.Latitude}, {training.Longitude}, {training.Description}, {training.FeedBack}");
            }

            Console.WriteLine("\tNotifications :");
            Console.WriteLine("\t---------------------------------");

            foreach (var notification in athlete.Notifications)
            {
                Console.WriteLine($"\t\t{notification.IdNotif} - {notification.Date}, {notification.Statut}, {notification.Message}");
            }
            
            Console.WriteLine("\tSources de données :");
            Console.WriteLine("\t---------------------------------");

            Console.WriteLine("\t\t" + (athlete.DataSource?.Model ?? "Aucune source de données"));
        }

        Console.WriteLine("---------------------------------\n");
    }

}